// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:rxdart/rxdart.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_cloud_messaging_bloc/src/core/exceptions/notification_exeption.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/repositories/notification_repository.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class ListenNotification extends StreamUseCase<NoParam, RemoteNotification> {
  final NotificationRepository _notificationRepository;

  ListenNotification({required NotificationRepository notificationRepository})
      : _notificationRepository = notificationRepository;
  @override
  FutureOrResult<Stream<RemoteNotification>> execute(NoParam? params) async {
    Stream<RemoteNotification>? notificationStream;

    final notificationStreamResponse =
        await _notificationRepository.onNotificationAccepted();

    if (notificationStreamResponse.isOk) {
      notificationStream = notificationStreamResponse.ok;
    } else if (notificationStreamResponse.isErr) {
      final notificationBackgroundStreamResponse =
          await _notificationRepository.onNotificationBackgroundAccepted();

      if (notificationBackgroundStreamResponse.isErr) {
        return Err(notificationBackgroundStreamResponse.err!);
      }

      final notificationForegroundStreamResponse =
          await _notificationRepository.onNotificationForegroundAccepted();

      if (notificationForegroundStreamResponse.isErr) {
        return Err(notificationBackgroundStreamResponse.err!);
      }

      notificationStream = Rx.merge([
        notificationBackgroundStreamResponse.ok!,
        notificationBackgroundStreamResponse.ok!,
      ]);
    }

    if (notificationStream == null) {
      return const Err(NotificationException('Cannot listen notifications'));
    }

    return Ok(notificationStream);
  }
}
